unit ColorPaletteEditor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, ColorPalette, Utility;

type
  TfrmColorPaletteEditor = class( TForm )
    pbPal: TPaintBox;
    Label1: TLabel;
    dfIndex: TPanel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    dfRed: TEdit;
    dfGreen: TEdit;
    dfBlue: TEdit;
    pbOK: TButton;
    pbModify: TButton;
    shapeCursor: TShape;
    pbCancel: TButton;
    ColorPalette1: TColorPalette;
    ColorDialog1: TColorDialog;
    pbLoad: TButton;
    pbSave: TButton;
    dlgOpen: TOpenDialog;
    dlgSave: TSaveDialog;
    procedure dfRedKeyPress(Sender: TObject; var Key: Char);
    procedure pbPalPaint(Sender: TObject);
    procedure pbPalMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormShow(Sender: TObject);
    procedure dfRedChange(Sender: TObject);
    procedure dfRedKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure pbModifyClick(Sender: TObject);
    procedure pbLoadClick(Sender: TObject);
    procedure pbSaveClick(Sender: TObject);
  private
    nIndex: integer;
    bCreated: boolean;
    function GetPalette: HPalette; override;
    procedure ModifyEntry;
  public
  end;

var
  frmColorPaletteEditor: TfrmColorPaletteEditor;

implementation

{$R *.DFM}

procedure TfrmColorPaletteEditor.dfRedKeyPress(Sender: TObject;
  var Key: Char);
begin
  if Key in [Chr( vk_Tab ),Chr( vk_Back )] then
     Exit;
  if ( Key < '0' ) or ( Key > '9' ) then
     Key := #0;
end;

function TfrmColorPaletteEditor.GetPalette: HPalette;
begin
  Result := ColorPalette1.Palette;
end;

procedure TfrmColorPaletteEditor.pbPalPaint(Sender: TObject);
var
  x, y: integer;
  rectFill: TRect;
begin
  with pbPal do
     begin
        SelectPalette( Canvas.Handle, ColorPalette1.Palette, FALSE );
        RealizePalette( Canvas.Handle );
        Canvas.Brush.Style := bsSolid;
        Canvas.Pen.Color := clWhite;
        Canvas.Pen.Width := 1;
        for x := 0 to 15 do
           for y := 0 to 15 do
              begin
                 Canvas.Brush.Color := PaletteIndex( y * 16 + x );
                 rectFill.Left := x * 16;
                 rectFill.Top := y * 16;
                 rectFill.Right := rectFill.Left + 15;
                 rectFill.Bottom := rectFill.Top + 15;
                 Canvas.FillRect( rectFill );
              end;
     end;
end;

procedure TfrmColorPaletteEditor.pbPalMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  x_, y_: integer;
begin
  x_ := X div 16;
  y_ := Y div 16;
  nIndex := y_ * 16 + x_;
  dfIndex.Caption := IntToStr( nIndex );
  dfRed.Text := IntToStr( ColorPalette1.PaletteEntry[nIndex].peRed );
  dfGreen.Text := IntToStr( ColorPalette1.PaletteEntry[nIndex].peGreen );
  dfBlue.Text := IntToStr( ColorPalette1.PaletteEntry[nIndex].peBlue );
  shapeCursor.Left := 8 + x_ * 16;
  shapeCursor.Top := 8 + y_ * 16;
end;

procedure TfrmColorPaletteEditor.FormShow(Sender: TObject);
begin
  if bCreated then
     Exit;
  nIndex := 0;
  pbPalMouseUp( self, mbLeft, [], 1, 1 );
  bCreated := TRUE;
end;

procedure TfrmColorPaletteEditor.dfRedChange(Sender: TObject);
begin
  with Sender as TEdit do
     begin
        if Text = '' then
           Tag := 0
        else
           begin
              Tag := StrToInt( Text );
              if Tag > 255 then
                 Text := '0'
           end;
     end;
end;

procedure TfrmColorPaletteEditor.ModifyEntry;
var
  pe: TPaletteEntry;
begin
  pe.peRed := dfRed.Tag;
  pe.peGreen := dfGreen.Tag;
  pe.peBlue := dfBlue.Tag;
  pe.peFlags := ColorPalette1.PaletteEntry[nIndex].peFlags;
  ColorPalette1.PaletteEntry[nIndex] := pe;
  ColorPalette1.Refresh;
  SelectPalette( pbPal.Canvas.Handle, ColorPalette1.Palette, FALSE );
  RealizePalette( pbPal.Canvas.Handle );
end;

procedure TfrmColorPaletteEditor.dfRedKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  ModifyEntry;
end;

procedure TfrmColorPaletteEditor.pbModifyClick(Sender: TObject);
var
  nRGB: longint;
begin
  with ColorDialog1 do
     begin
        Color := RGB( dfRed.Tag, dfGreen.Tag, dfBlue.Tag );
        if Execute then
           begin
              nRGB := ColorToRGB( Color );
              dfRed.Text := IntToStr( GetRValue( nRGB ) );
              dfGreen.Text := IntToStr( GetGValue( nRGB ) );
              dfBlue.Text := IntToStr( GetBValue( nRGB ) );
              ModifyEntry;
           end;
     end;
end;

procedure TfrmColorPaletteEditor.pbLoadClick(Sender: TObject);
var
  strOpen: TStrings;
  i: integer;
  sLine, sEntry: string;
begin
  if dlgOpen.Execute then
     begin
        dlgSave.FileName := dlgOpen.FileName;
        strOpen := TStringList.Create;
        strOpen.LoadFromFile( dlgOpen.FileName );
        for i := 0 to 255 do
           begin
              sLine := strOpen[i + 3];
              sEntry := GetToken( sLine, ' ' ) + ',' +
                 GetToken( sLine, ' ' ) + ',' +
                 GetToken( sLine, ' ' ) + ',' + IntToStr( PC_NOCOLLAPSE );
              ColorPalette1.PaletteEntries[i] := sEntry;
           end;
        ColorPalette1.Refresh;
        dfRed.Text := IntToStr( ColorPalette1.PaletteEntry[nIndex].peRed );
        dfGreen.Text := IntToStr( ColorPalette1.PaletteEntry[nIndex].peGreen );
        dfBlue.Text := IntToStr( ColorPalette1.PaletteEntry[nIndex].peBlue );
        ModifyEntry;
        strOpen.Free;
     end;
end;

procedure TfrmColorPaletteEditor.pbSaveClick(Sender: TObject);
var
  strSave: TStrings;
  i: integer;
  sLine, sEntry: string;
begin
  if dlgSave.Execute then
     begin
        strSave := TStringList.Create;
        strSave.Add( 'JASC-PAL' );
        strSave.Add( '0100' );
        strSave.Add( '256' );
        for i := 0 to 255 do
           begin
              sLine := ColorPalette1.PaletteEntries[i];
              sEntry := GetToken( sLine, ',' ) + ' ' +
                 GetToken( sLine, ',' ) + ' ' +
                 GetToken( sLine, ',' );
              strSave.Add( sEntry );
           end;
        strSave.SaveToFile( dlgSave.FileName );
        strSave.Free;
     end;
end;

end.
